// ##########################################################################
// ####                                                                  ####
// ####      WarpEd - A portable text editor for OS/2 Warp and Linux     ####
// ####    ===========================================================   ####
// ####                                                                  ####
// #### edit.h                                                           ####
// ####                                                                  ####
// #### Version 0.80  --  October 06, 2000                               ####
// ####                                                                  ####
// #### Copyright (C) 1996  Thomas Dreibholz                             ####
// ####               1997  Molbachweg 7                                 ####
// ####                     51674 Wiehl/Germany                          ####
// ####                     EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef EDIT_H
#define EDIT_H


#include "terminal.h"
#include "row.h"
#include "lines.h"


#include <bool.h>
#include <fstream.h>


// Editor-Klasse
class Editor
{
   // ===== Editor-Daten =================================================
   public:

   // ----- Positionen auf dem Display ----------
   int leftEdge, topEdge;
   int width,height;
   int cursorX, cursorY;
   int offsetX, offsetY;   
   int maxRows;
   int maxCols;        

   // ----- Status ------------------------------
   enum EdStatus {    ok = 0,
                      topOfText   = 1,  endOfText  = 2,
                      lineTooLong = 3,  bufferFull = 4,
                      ioError     = 8};
   enum BlockStatus { noBlock = 0, blockStart = 1, blockEnd = 2};
   
   EdStatus          status;
   BlockStatus       blockStatus;
   bool              changed;
   bool              isGood;
   
   int               markX1,markY1,
                     markX2,markY2;
   char*             tmpName;
 
   // ----- Einstellungen -----------------------
   bool              autoBackup;
   bool              autoCursorJump;  
   bool              autoIndent;
   bool              overwriteBlocks;
   bool              readOnly;
   int               fastScrollX,
                     fastScrollY;

   // ----- Unterklassen ------------------------
   LineManager       lineMgr;
   EdLine            edLine;
   TerminalClass*    Terminal;

   
   // ===== Initialisierung ==============================================
   public:
   Editor();
   void init(TerminalClass *term,
          int x, int y, int width, int height,
          int mr, int mc,
          char* tempName);
    ~Editor();
   inline bool Good() const;
   inline bool Bad()  const;

   void        Resize(int x, int y, int w, int h);

   // ===== Dateioperationen =============================================
   void        NewText();
   bool        LoadText(char* name);
   bool        LoadText(ifstream& is);
   bool        SaveText(char* name);
   bool        SaveText(ofstream& os);

   // ===== Texteingabe ==================================================
   bool        InsertChar(char chr);
   void        Tab(int i);
   void        Return();
   void        Del();
   void        ShiftDel();
   void        Backspace();
   void        ShiftBackspace();
 
   // ===== Cursorbewegungen =============================================
   void        CursorHome();
   void        CursorEnd();
   void        CursorLeft();
   void        CursorRight();
   void        CursorUp();
   void        CursorDown();
   inline void PageUp();
   inline void PageDown();
   void        ScrollUp(int subCursor, int subOffset);
   void        ScrollDown(int addCursor, int addOffset);
   void        JumpToTop();
   void        JumpToEnd();
   void        SetCursor(int y, int x);
   inline void ShowCursor();
   inline void HideCursor();

   // ===== Markieren ====================================================
   void        BlockStart();
   void        BlockEnd();
   inline void BlockCursorHome();
   inline void BlockCursorEnd();
   inline void BlockCursorLeft();
   inline void BlockCursorRight();
   inline void BlockCursorUp();
   inline void BlockCursorDown();
   inline void BlockPageUp();
   inline void BlockPageDown();
   
   // ===== Blockoperationen =============================================
   bool CopyBlock(char* name);
   bool CopyBlock(ofstream& os); 
   bool CutBlock(char* name);
   bool CutBlock(ofstream& os);
   bool InsertBlock(char* name);
   bool InsertBlock(istream& is);
   void KillBlock();

   // ===== Refresh des Displays =========================================
   void RefreshDisplay(int start, int end);
   void UpdateDisplay();

   // ===== Daten abfragen ===============================================
   inline EdStatus Status();
   inline int CursorX() const;
   inline int CursorY() const;
   inline int OffsetX() const;
   inline int OffsetY() const;
   inline int Count()   const;
   inline bool ReadOnly()        const;
   inline bool AutoBackup()      const;
   inline bool Changed()         const;
   inline bool AutoIndent()      const;
   inline bool OverwriteBlocks() const;

   // ===== Zeilenoperationen ============================================
   void       GetLine(int number, char* text);
   void       SetLine(int number, char* text);  

   // ===== Optionen einstellen ==========================================
   inline void SetAutoBackup(bool on);        // Automatisch Backup erzeugen
   inline void SetAutoCursorJump(bool on);    // Cursor bei Textende in nchste Zeile   
   inline void SetAutoIndent(bool on);        // Automatisch einrcken
   inline void SetFastScroll(int divisorX, int divisorY);  // Schnelles Scrollen
   inline void SetOverwriteBlocks(bool on);   // Blcke berschreiben
   inline void SetReadOnly(bool on);          // Read-only-Modus

   // ===== interne Funktionen ===========================================
   private:

   // Editierzeile updaten
   void   UpdateEditLine();

   // Aktuelle Editierzeile in LineManager schreiben   
   inline void SaveCurrentLine();
   
   // Aktuelle Zeile aus LineManager in Editierzeile laden
   inline void LoadCurrentLine();     
   
   // Aktuelle X-Position aus Editierzeile bernehmen und ggf. Display updaten
   void        RethinkXPosition();
   
   // Markierung aufheben, Display ggf. updaten
   void   UnmarkBlock(int update);

   // Markierung mit Cursortasten: Markierung fortsetzen oder ggf. neu starten
   void   CheckBlock();

   // Block falls notwendig berschreiben
   int    OverwriteBlock();
 
   private:
   int    blockStartX,blockStartY,
          blockEndX,  blockEndY;
   int    skipUpdate;                   
};
#endif
